# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Anaconda Nucleus login dialogs."""

__all__ = ['NucleusLoginPage']

import typing
import requests
from qtpy.QtCore import Qt
from qtpy.QtWidgets import QApplication, QWidget
from anaconda_navigator.api import nucleus as nucleus_api
from anaconda_navigator.config import CONF, NUCLEUS_BRAND
from anaconda_navigator.static.images import ANACONDA_NUCLEUS_LOGIN_LOGO
from .base_dialogs import BaseLoginPage
from .utils import TextContainer


NUCLEUS_LOGIN_TEXT_CONTAINER = TextContainer(
    title='Sign In',
    info_frame_text=(
        'Log into Anaconda Nucleus to backup your environment and access more data science tools. If you don’t have '
        'an account, click <a href="{}" style="color:#43B049;text-decoration: none">here</a>.'
    ),
    form_forgot_links_msg=(
        'Forget your <a href="{password_url}" style="color:#43B049; text-decoration:none">password</a>?'
    ),
    message_box_error_text='The Anaconda Nucleus API domain is not specified! Please, set in preferences.',
    header_frame_logo_path=ANACONDA_NUCLEUS_LOGIN_LOGO,
)


class NucleusLoginPage(BaseLoginPage):

    def __init__(self, parent: typing.Optional[QWidget] = None) -> None:
        """Initialize new :class:`~NucleusLoginPage` instance."""
        super().__init__(nucleus_api.NucleusAPI(), NUCLEUS_LOGIN_TEXT_CONTAINER, parent=parent)
        self.label_login.setText('Email')
        self.text_login.setValidator(None)

        self.api_url_config_option = 'nucleus_base_url'
        self.brand = NUCLEUS_BRAND

        self.finished.connect(self.__finished)

    def login(self):
        base_url = CONF.get('main', self.api_url_config_option)
        if not base_url:
            self._show_message_box(title='Login Error', text=self.text_container.domain_not_found_msg)
            return

        self.button_login.setEnabled(False)
        QApplication.setOverrideCursor(Qt.WaitCursor)

        self.label_message.setText('')
        self.label_message.setVisible(False)

        text_login: str = self.text_login.text().lower()
        self.text_login.setText(text_login)

        worker: nucleus_api.TaskWorker = self.api.login.worker(
            username=text_login,
            password=self.text_password.text(),
        )
        self.finished.connect(worker.cancel)
        worker.signals.sig_done.connect(lambda result: self.finished.disconnect(worker.cancel))
        worker.signals.sig_done.connect(self.__process_login_response)
        worker.start()

    def __process_login_response(self, result: nucleus_api.TaskResult) -> None:
        """Check response from Nucleus server for login."""
        if not self.isVisible():
            return

        self.button_login.setDisabled(False)
        self.check_text()
        QApplication.restoreOverrideCursor()

        error_msg: typing.Optional[str] = None
        if result.status != nucleus_api.TaskStatus.SUCCEEDED:
            error: BaseException = result.exception
            error_msg = str(error)
            if isinstance(error, requests.RequestException) and (error.response is not None):
                if error.response.status_code in (401, 403):
                    error_msg = 'Invalid credentials'

        if self.api.token.valid:
            self.accept()
            return

        if error_msg is not None:
            self.label_message.setText(error_msg)
            self.label_message.setVisible(True)

            url: str = CONF.get('main', self.api_url_config_option, '-')
            username: str = result.call.kwargs['username']
            self.tracker.track_event('authenticate', 'login failed', label=f'{url}/{username}: {error_msg.lower()}')

            self.text_password.setFocus()
            self.text_password.selectAll()

    def __finished(self) -> None:
        """Update dialog state when it is closed."""
        QApplication.restoreOverrideCursor()

    def update_links(self):
        """Update links."""

        base_url = CONF.get('main', 'nucleus_base_url', None)

        if not base_url:
            return

        base_url = base_url.rstrip('/')
        forgot_links_updated_text = self.forgot_links.text().format(
            password_url=base_url + '/forgot-password'
        )
        info_updated_text = self.label_information.text().format(base_url + '/sign-up')
        self.label_information.setText(info_updated_text)
        self.forgot_links.setText(forgot_links_updated_text)

        self.label_information.linkActivated.connect(lambda activated_link: self.open_url(activated_link))
        self.forgot_links.linkActivated.connect(lambda activated_link: self.open_url(activated_link))



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Environments Tab."""

# yapf: disable

from __future__ import absolute_import, division, print_function

# Third party imports
from qtpy.QtCore import QPoint, Qt, Signal
from qtpy.QtWidgets import QHBoxLayout, QMenu, QVBoxLayout

# Local imports
from anaconda_navigator.api.anaconda_api import AnacondaAPI
from anaconda_navigator.utils import constants as C
from anaconda_navigator.utils import launch
from anaconda_navigator.utils.logs import logger
from anaconda_navigator.utils.styles import load_style_sheet
from anaconda_navigator.widgets import (ButtonToolNormal,
                                        FrameEnvironmentsList,
                                        FrameEnvironmentsPackages,
                                        FrameTabHeader, WidgetBase)
from anaconda_navigator.widgets.helperwidgets import (ButtonToggleCollapse,
                                                      LineEditSearch)
from anaconda_navigator.widgets.lists.environments import (ListItemEnv,
                                                           ListWidgetEnv)
from anaconda_navigator.widgets.manager.packages import CondaPackagesWidget


# yapf: enable


class EnvironmentsTab(WidgetBase):
    """Conda environments tab."""
    BLACKLIST = ['anaconda-navigator', '_license']  # Hide in package manager

    # --- Signals
    # -------------------------------------------------------------------------
    sig_ready = Signal()

    # name, prefix, sender
    sig_item_selected = Signal(object, object, object)

    # sender, func_after_dlg_accept, func_callback_on_finished
    sig_create_requested = Signal()
    sig_clone_requested = Signal()
    sig_backup_requested = Signal()
    sig_import_requested = Signal()
    sig_remove_requested = Signal()

    # button_widget, sender_constant
    sig_channels_requested = Signal(object, object)

    # sender_constant
    sig_update_index_requested = Signal(object)
    sig_cancel_requested = Signal(object)

    # conda_packages_action_dict, pip_packages_action_dict
    sig_packages_action_requested = Signal(object, object)

    def __init__(self, parent=None):
        """Conda environments tab."""
        super(EnvironmentsTab, self).__init__(parent)

        # Variables
        self.api = AnacondaAPI()
        self.current_prefix = None
        self.style_sheet = None

        # Widgets
        self.frame_header_left = FrameTabHeader()
        self.frame_list = FrameEnvironmentsList(self)
        self.frame_widget = FrameEnvironmentsPackages(self)
        self.text_search = LineEditSearch()
        self.list = ListWidgetEnv()
        self.menu_list = QMenu()
        self.button_create = ButtonToolNormal(text="Create")
        self.button_clone = ButtonToolNormal(text="Clone")
        self.button_import = ButtonToolNormal(text="Import")
        self.button_backup = ButtonToolNormal(text="Backup")
        self.button_remove = ButtonToolNormal(text="Remove")
        self.button_toggle_collapse = ButtonToggleCollapse()
        self.widget = CondaPackagesWidget(parent=self)

        # Widgets setup
        self.frame_list.is_expanded = True
        self.text_search.setPlaceholderText("Search Environments")
        self.list.setContextMenuPolicy(Qt.CustomContextMenu)
        self.button_create.setObjectName("create")  # Needed for QSS selectors
        self.button_clone.setObjectName("clone")
        self.button_import.setObjectName("import")
        self.button_backup.setObjectName("backup")
        self.button_remove.setObjectName("remove")
        self.widget.textbox_search.set_icon_visibility(False)

        # Layouts
        layout_header_left = QVBoxLayout()
        layout_header_left.addWidget(self.text_search)
        self.frame_header_left.setLayout(layout_header_left)

        layout_buttons = QHBoxLayout()
        layout_buttons.addWidget(self.button_create)
        layout_buttons.addWidget(self.button_clone)
        layout_buttons.addWidget(self.button_import)
        layout_buttons.addWidget(self.button_backup)
        layout_buttons.addWidget(self.button_remove)

        layout_list_buttons = QVBoxLayout()
        layout_list_buttons.addWidget(self.frame_header_left)
        layout_list_buttons.addWidget(self.list)
        layout_list_buttons.addLayout(layout_buttons)
        self.frame_list.setLayout(layout_list_buttons)

        layout_widget = QHBoxLayout()
        layout_widget.addWidget(self.widget)
        self.frame_widget.setLayout(layout_widget)

        layout_main = QHBoxLayout()
        layout_main.addWidget(self.frame_list, 10)
        layout_main.addWidget(self.button_toggle_collapse, 1)
        layout_main.addWidget(self.frame_widget, 30)

        self.setLayout(layout_main)

        # Signals for buttons and boxes
        self.button_toggle_collapse.clicked.connect(self.expand_collapse)
        self.button_create.clicked.connect(self.sig_create_requested)
        self.button_clone.clicked.connect(self.sig_clone_requested)
        self.button_import.clicked.connect(self.sig_import_requested)
        self.button_backup.clicked.connect(self.sig_backup_requested)
        self.button_remove.clicked.connect(self.sig_remove_requested)
        self.text_search.textChanged.connect(self.filter_list)

        # Signals for list
        self.list.sig_item_selected.connect(self._item_selected)

        # Signals for packages widget
        self.widget.sig_ready.connect(self.sig_ready)
        self.widget.sig_channels_requested.connect(self.sig_channels_requested)
        self.widget.sig_update_index_requested.connect(self.sig_update_index_requested)
        self.widget.sig_cancel_requested.connect(self.sig_cancel_requested)
        self.widget.sig_packages_action_requested.connect(self.sig_packages_action_requested)

    # --- Setup methods
    # -------------------------------------------------------------------------
    def setup(self, conda_data):
        """Setup tab content and populates the list of environments."""
        self.set_widgets_enabled(False)
        conda_processed_info = conda_data.get('processed_info')
        environments = conda_processed_info.get('__environments')
        packages = conda_data.get('packages')
        self.current_prefix = conda_processed_info.get('default_prefix')
        self.set_environments(environments)
        self.set_packages(packages)

    def set_environments(self, environments):
        """Populate the list of environments."""
        self.list.clear()
        selected_item_row = 0
        for i, (env_prefix, env_name) in enumerate(environments.items()):
            item = ListItemEnv(prefix=env_prefix, name=env_name)
            item.button_options.clicked.connect(self.show_environment_menu)
            if env_prefix == self.current_prefix:
                selected_item_row = i
            self.list.addItem(item)

        self.list.setCurrentRow(selected_item_row, loading=True)
        self.filter_list()
        self.list.scrollToItem(self.list.item(selected_item_row))

    def _set_packages(self, worker, output, error):
        """Set packages callback."""
        packages, model_data = output
        self.widget.setup(packages, model_data)
        self.set_widgets_enabled(True)
        self.set_loading(prefix=self.current_prefix, value=False)

    def set_packages(self, packages):
        """Set packages widget content."""
        worker = self.api.process_packages(packages, prefix=self.current_prefix, blacklist=self.BLACKLIST)
        worker.sig_chain_finished.connect(self._set_packages)

    def show_environment_menu(self, value=None, position=None):
        """Show the environment actions menu."""
        self.menu_list.clear()
        menu_item = self.menu_list.addAction('Open Terminal')
        menu_item.triggered.connect(lambda: self.open_environment_in('terminal'))

        for word in ['Python', 'IPython', 'Jupyter Notebook']:
            menu_item = self.menu_list.addAction("Open with " + word)
            menu_item.triggered.connect(lambda x, w=word: self.open_environment_in(w.lower()))

        current_item = self.list.currentItem()
        prefix = current_item.prefix

        if isinstance(position, bool) or position is None:
            width = current_item.button_options.width()
            position = QPoint(width, 0)

        point = QPoint(0, 0)
        parent_position = current_item.button_options.mapToGlobal(point)
        self.menu_list.move(parent_position + position)

        # Disabled actions depending on the environment installed packages
        actions = self.menu_list.actions()
        actions[2].setEnabled(launch.check_prog('ipython', prefix))
        actions[3].setEnabled(launch.check_prog('notebook', prefix))

        self.menu_list.exec_()

    def open_environment_in(self, which):
        """Open selected environment in console terminal."""
        prefix = self.list.currentItem().prefix
        logger.debug("%s, %s", which, prefix)

        if which == 'terminal':
            launch.console(prefix)
        else:
            launch.py_in_console(prefix, which)

    # --- Common Helpers (# FIXME: factor out to common base widget)
    # -------------------------------------------------------------------------
    def _item_selected(self, item):
        """Callback to emit signal as user selects an item from the list."""
        self.set_loading(prefix=item.prefix)
        self.sig_item_selected.emit(item.name, item.prefix, C.TAB_ENVIRONMENT)

    def add_temporal_item(self, name):
        """Creates a temporal item on list while creation becomes effective."""
        item_names = [item.name for item in self.list.items()]
        item_names.append(name)
        index = list(sorted(item_names)).index(name) + 1
        item = ListItemEnv(name=name)
        self.list.insertItem(index, item)
        self.list.setCurrentRow(index)
        self.list.scrollToItem(item)
        item.set_loading(True)

    def expand_collapse(self):
        """Expand or collapse the list selector."""
        if self.frame_list.is_expanded:
            self.frame_list.hide()
            self.frame_list.is_expanded = False
        else:
            self.frame_list.show()
            self.frame_list.is_expanded = True

    def filter_list(self, text=None):
        """Filter items in list by name."""
        text = self.text_search.text().lower()
        for i in range(self.list.count()):
            item = self.list.item(i)
            item.setHidden(text not in item.name.lower())

            if not item.widget.isVisible():
                item.widget.repaint()

    def ordered_widgets(self, next_widget=None):
        """Return a list of the ordered widgets."""
        if next_widget is not None:
            self.widget.table_last_row.add_focus_widget(next_widget)

        ordered_widgets = [self.text_search]
        ordered_widgets += self.list.ordered_widgets()
        ordered_widgets += [
            self.button_create,
            self.button_clone,
            self.button_import,
            self.button_backup,
            self.button_remove,
            self.widget.combobox_filter,
            self.widget.button_channels,
            self.widget.button_update,
            self.widget.textbox_search,
            # self.widget.table_first_row,
            self.widget.table,
            self.widget.table_last_row,
            self.widget.button_apply,
            self.widget.button_clear,
            self.widget.button_cancel,
        ]
        return ordered_widgets

    def refresh(self):
        """Refresh the enabled/disabled status of the widget and subwidgets."""
        is_root = self.current_prefix == self.api.ROOT_PREFIX
        self.button_clone.setDisabled(is_root)
        self.button_remove.setDisabled(is_root)

    def set_loading(self, prefix=None, value=True):
        """Set the item given by `prefix` to loading state."""
        for row, item in enumerate(self.list.items()):
            if item.prefix == prefix:
                item.set_loading(value)
                self.list.setCurrentRow(row)
                break

    def set_widgets_enabled(self, value):
        """Change the enabled status of widgets and subwidgets."""
        self.list.setEnabled(value)
        self.button_create.setEnabled(value)
        self.button_clone.setEnabled(value)
        self.button_import.setEnabled(value)
        self.button_backup.setEnabled(value)
        self.button_remove.setEnabled(value)
        self.widget.set_widgets_enabled(value)
        if value:
            self.refresh()

    def update_status(self, action='', message='', value=None, max_value=None):
        """Update widget status and progress bar."""
        self.widget.update_status(action=action, message=message, value=value, max_value=max_value)

    def update_style_sheet(self, style_sheet=None):
        """Update custom CSS stylesheet."""
        if style_sheet is None:
            self.style_sheet = load_style_sheet()
        else:
            self.style_sheet = style_sheet

        self.setStyleSheet(self.style_sheet)
        self.list.update_style_sheet(self.style_sheet)
        self.menu_list.setStyleSheet(self.style_sheet)


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run local test."""
    from anaconda_navigator.utils.qthelpers import qapplication
    from anaconda_navigator.utils.styles import load_style_sheet

    app = qapplication()
    widget = EnvironmentsTab()
    widget.setStyleSheet(load_style_sheet())
    # widget.setup()
    widget.show()
    app.exec_()


if __name__ == '__main__':  # pragma: no cover
    local_test()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Tests for quit-like dialogs."""

# yapf: disable


# Third party imports
from qtpy.QtCore import Qt
import pytest

# Local imports
from anaconda_navigator.utils.fixtures import tmpconfig
from anaconda_navigator.widgets.dialogs.quit import (ClosePackageManagerDialog,
                                                     QuitApplicationDialog,
                                                     QuitBusyDialog,
                                                     QuitRunningAppsDialog)


# yapf: enable

tmpconfig
RUNNING_PROCESSES = [
    {
        'pids': [12, 123, 412],
        'package': 'test-app-1',
        'command': 'some command',
        'prefix': '/some/prefix',
    },
    {
        'pids': [345, 6788, 4112],
        'package': 'test-app-2',
        'command': 'some-command-2',
        'prefix': '/some/prefix',
    },
    {
        'pids': [345, 6788, 4112],
        'package': 'test-app-3',
        'command': 'some-command-3',
        'prefix': '/some/prefix',
    },
]
APP_IN_LIST = RUNNING_PROCESSES[0]['package']


# --- Fixtures
# -----------------------------------------------------------------------------
@pytest.fixture
def botquittrue(qtbot, tmpconfig):
    tmpconfig.set('main', 'hide_quit_dialog', True)
    widget = QuitApplicationDialog(config=tmpconfig)
    widget.show()
    qtbot.addWidget(widget)
    qtbot.config = tmpconfig
    return qtbot, widget


@pytest.fixture
def botquitfalse(qtbot, tmpconfig):
    tmpconfig.set('main', 'hide_quit_dialog', False)
    widget = QuitApplicationDialog(config=tmpconfig)
    widget.show()
    qtbot.addWidget(widget)
    qtbot.config = tmpconfig
    return qtbot, widget


@pytest.fixture
def botquitbusy(qtbot):
    widget = QuitBusyDialog()
    widget.show()
    qtbot.addWidget(widget)
    qtbot.config = tmpconfig
    return qtbot, widget


@pytest.fixture
def closecondawidget(qtbot):
    widget = ClosePackageManagerDialog()
    widget.show()
    qtbot.addWidget(widget)
    return qtbot, widget


@pytest.fixture
def botquitrunningappshide(qtbot, tmpconfig):
    tmpconfig.set('main', 'running_apps_to_close', [APP_IN_LIST])
    tmpconfig.set('main', 'hide_running_apps_dialog', True)
    widget = QuitRunningAppsDialog(running_processes=RUNNING_PROCESSES, config=tmpconfig)
    widget.show()
    qtbot.addWidget(widget)
    qtbot.config = tmpconfig
    return qtbot, widget


@pytest.fixture
def botquitrunningappsshow(qtbot, tmpconfig):
    tmpconfig.set('main', 'running_apps_to_close', [APP_IN_LIST])
    tmpconfig.set('main', 'hide_running_apps_dialog', False)
    widget = QuitRunningAppsDialog(running_processes=RUNNING_PROCESSES, config=tmpconfig)
    widget.show()
    qtbot.addWidget(widget)
    qtbot.config = tmpconfig
    return qtbot, widget


# --- Tests
# -----------------------------------------------------------------------------
class TestQuitDialog:
    def test_hide(self, botquitfalse):
        qtbot, widget = botquitfalse
        assert widget.checkbox.checkState() == Qt.Unchecked

    def test_show(self, botquittrue):
        qtbot, widget = botquittrue
        assert widget.checkbox.checkState() == Qt.Checked

    def test_accept(self, botquitfalse):
        qtbot, widget = botquitfalse
        qtbot.mouseClick(widget.checkbox, Qt.LeftButton)
        with qtbot.waitSignal(widget.accepted, 1000, True):
            qtbot.mouseClick(widget.button_ok, Qt.LeftButton)
        assert widget.config.get('main', 'hide_quit_dialog')

    def test_reject(self, botquittrue):
        qtbot, widget = botquittrue
        qtbot.mouseClick(widget.checkbox, Qt.LeftButton)
        with qtbot.waitSignal(widget.rejected, 1000, True):
            qtbot.mouseClick(widget.button_cancel, Qt.LeftButton)
        assert widget.config.get('main', 'hide_quit_dialog')


class TestQuitBusyDialog:
    def test_accept(self, botquitbusy):
        qtbot, widget = botquitbusy
        with qtbot.waitSignal(widget.accepted, 1000, True):
            qtbot.mouseClick(widget.button_ok, Qt.LeftButton)

    def test_reject(self, botquitbusy):
        qtbot, widget = botquitbusy
        with qtbot.waitSignal(widget.rejected, 1000, True):
            qtbot.mouseClick(widget.button_cancel, Qt.LeftButton)


class TestCloseCondaDialog:
    def test_ok(self, closecondawidget):
        qtbot, widget = closecondawidget
        with qtbot.waitSignal(widget.accepted, 1000, True):
            qtbot.mouseClick(widget.button_ok, Qt.LeftButton)

    def test_cancel(self, closecondawidget):
        qtbot, widget = closecondawidget
        with qtbot.waitSignal(widget.rejected, 1000, True):
            qtbot.mouseClick(widget.button_cancel, Qt.LeftButton)


class TestQuitRunningAppsDialog:
    def test_hide(self, botquitrunningappshide):
        qtbot, widget = botquitrunningappshide
        assert widget.checkbox.checkState() == Qt.Checked

    def test_show(self, botquitrunningappsshow):
        qtbot, widget = botquitrunningappsshow
        assert widget.checkbox.checkState() == Qt.Unchecked

    def test_accept(self, botquitrunningappsshow):
        qtbot, widget = botquitrunningappsshow
        item = widget.list.item(0)
        item.set_checked(False)
        with qtbot.waitSignal(widget.accepted, 5000, True):
            qtbot.mouseClick(widget.button_close, Qt.LeftButton)
        apps = qtbot.config.get('main', 'running_apps_to_close')
        assert apps == []

    def test_accept_2(self, botquitrunningappsshow):
        qtbot, widget = botquitrunningappsshow
        item = widget.list.item(1)
        item.set_checked(True)
        with qtbot.waitSignal(widget.accepted, 5000, True):
            qtbot.mouseClick(widget.button_close, Qt.LeftButton)
        apps = qtbot.config.get('main', 'running_apps_to_close')
        assert apps == ['test-app-1', 'test-app-2']

    def test_reject(self, botquitrunningappsshow):
        qtbot, widget = botquitrunningappsshow
        item = widget.list.item(2)
        item.set_checked(True)
        with qtbot.waitSignal(widget.rejected, 5000, True):
            qtbot.mouseClick(widget.button_cancel, Qt.LeftButton)
        apps = qtbot.config.get('main', 'running_apps_to_close')
        assert apps == ['test-app-1']



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Dialogs for conflicting situations."""

__all__ = ['ConflictDialog']

from qtpy import QtCore
from qtpy import QtWidgets

from anaconda_navigator import widgets
from . import common


class ConflictDialog(common.EnvironmentActionsDialog):
    """Create new environment dialog if navigator conflicts with deps."""

    def __init__(self, parent=None, package=None, extra_message='', current_prefix=None):
        """Create new environment dialog if navigator conflicts with deps."""
        super(ConflictDialog, self).__init__(parent=parent)

        parts = package.split('=')
        self.package = parts[0] if '=' in package else package
        self.package_version = parts[-1] if '=' in package else ''
        self.current_prefix = current_prefix

        base_message = '<b>{0}</b> cannot be installed on this ' 'environment.'.format(package)

        base_message = extra_message or base_message
        # Widgets
        self.label_info = widgets.LabelBase(
            base_message + '<br><br>'
            'Do you want to install the package in an existing '
            'environment or <br>create a new environment?'
        )
        self.label_name = widgets.LabelBase('Name:')
        self.label_prefix = widgets.LabelBase(' ' * 100)
        self.label_location = widgets.LabelBase('Location:')
        self.combo_name = widgets.ComboBoxBase()
        self.button_ok = widgets.ButtonPrimary('Create')
        self.button_cancel = widgets.ButtonNormal('Cancel')

        # Widgets setup
        self.align_labels([self.label_name, self.label_location])
        self.combo_name.setEditable(True)
        self.combo_name.setCompleter(None)
        self.setMinimumWidth(self.BASE_DIALOG_WIDTH)
        self.setWindowTitle("Create new environment for '{}'".format(package))
        self.label_prefix.setObjectName('environment-location')
        self.combo_name.setObjectName('environment-selection')

        # Layouts
        grid_layout = QtWidgets.QGridLayout()
        grid_layout.addWidget(self.label_name, 0, 0)
        grid_layout.addWidget(widgets.SpacerHorizontal(), 0, 1)
        grid_layout.addWidget(self.combo_name, 0, 2)
        grid_layout.addWidget(widgets.SpacerVertical(), 1, 0)
        grid_layout.addWidget(self.label_location, 2, 0)
        grid_layout.addWidget(widgets.SpacerHorizontal(), 2, 1)
        grid_layout.addWidget(self.label_prefix, 2, 2)

        layout_buttons = QtWidgets.QHBoxLayout()
        layout_buttons.addStretch()
        layout_buttons.addWidget(self.button_cancel)
        layout_buttons.addWidget(widgets.SpacerHorizontal())
        layout_buttons.addWidget(self.button_ok)

        main_layout = QtWidgets.QVBoxLayout()
        main_layout.addWidget(self.label_info)
        main_layout.addWidget(widgets.SpacerVertical())
        main_layout.addWidget(widgets.SpacerVertical())
        main_layout.addLayout(grid_layout)
        main_layout.addWidget(widgets.SpacerVertical())
        main_layout.addWidget(widgets.SpacerVertical())
        main_layout.addLayout(layout_buttons)

        self.setLayout(main_layout)

        # Signals
        self.button_ok.clicked.connect(self.accept)
        self.button_cancel.clicked.connect(self.reject)
        self.combo_name.setCurrentText(self.package)
        self.combo_name.currentTextChanged.connect(self.refresh)
        self.button_ok.setDisabled(True)

    def new_env_name(self):
        """Generate a unique environment name."""
        pkg_name_version = self.package + '-' + self.package_version
        if self.environments:
            if self.package not in self.environments.values():
                env_name = self.package
            elif pkg_name_version not in self.environments.values():
                env_name = pkg_name_version
            else:
                for i in range(1, 1000):
                    new_pkg_name = pkg_name_version + '_' + str(i)
                    if new_pkg_name not in self.environments.values():
                        env_name = new_pkg_name
                        break
        else:
            env_name = self.package
        return env_name

    def setup(self, worker, info, error):
        """Setup the dialog conda information as a result of a worker."""
        super(ConflictDialog, self).setup(worker, info, error)
        self.combo_name.blockSignals(True)
        self.combo_name.clear()
        new_env_name = self.new_env_name()
        for i, (env_prefix, env_name) in enumerate(self.environments.items()):
            # Do not include the env where the conflict was found!
            if self.current_prefix != env_prefix:
                self.combo_name.addItem(env_name, env_prefix)
                self.combo_name.setCurrentText(new_env_name)
                self.combo_name.setItemData(i, env_prefix, QtCore.Qt.ToolTipRole)
        self.combo_name.setCurrentText(new_env_name)
        self.combo_name.blockSignals(False)
        self.refresh()

    def refresh(self):
        """Refresh state of buttons based on content."""
        self.update_location()

        if self.environments:
            self.button_ok.setEnabled(bool(self.name))

    @property
    def name(self):
        """Return the name of the environment."""
        return self.combo_name.currentText()



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Styles for the application."""

# yapf: disable

# Standard library imports
import ast
import os
import re
import string

# Local imports
from anaconda_navigator.config import DEV
from anaconda_navigator.static import images
from anaconda_navigator.static.css import GLOBAL_SASS_STYLES_PATH, GLOBAL_STYLES_PATH


# This is a development dependency to parse the sass style sheet
try:
    import sass
    sass_module_installed = True
except Exception:
    sass_module_installed = False

# yapf: enable

BLUR_SIZE = 10
MANAGER_TABLE_STYLES = {
    'icon.action.not_installed': images.MANAGER_NOT_INSTALLED,
    'icon.action.installed': images.MANAGER_INSTALLED,
    'icon.action.remove': images.MANAGER_REMOVE,
    'icon.action.add': images.MANAGER_ADD,
    'icon.action.upgrade': images.MANAGER_UPGRADE,
    'icon.action.downgrade': images.MANAGER_DOWNGRADE,
    'icon.upgrade.arrow': images.MANAGER_UPGRADE_ARROW,
    'icon.spacer': images.MANAGER_SPACER,
    'icon.python': images.PYTHON_LOGO,
    'icon.anaconda': images.ANACONDA_LOGO,
    'color.background.remove': '#0000',
    'color.background.install': '#0000',
    'color.background.upgrade': '#0000',
    'color.background.downgrade': '#0000',
    'color.foreground.not.installed': "#666",
    'color.foreground.upgrade': "#00A3E0",
    'size.icons': (32, 32),
}


class SassVariables(object):
    """Enum to hold SASS defined variables."""
    def __init__(self):
        """Enum to hold SASS defined variables."""
        self.SHADOW_BLUR_RADIUS = 7  # Used for dialogs
        self.WIDGET_APPLICATION_TOTAL_HEIGHT = 200
        self.WIDGET_APPLICATION_TOTAL_WIDTH = 200
        self.WIDGET_CONTENT_PADDING = 5
        self.WIDGET_CONTENT_TOTAL_HEIGHT = 200
        self.WIDGET_CONTENT_TOTAL_WIDTH = 200
        self.WIDGET_CONTENT_PADDING = 5
        self.WIDGET_CONTENT_MARGIN = 5
        self.WIDGET_ENVIRONMENT_TOTAL_HEIGHT = 50
        self.WIDGET_IMPORT_ENVIRONMENT_TOTAL_HEIGHT = 55
        self.WIDGET_ENVIRONMENT_TOTAL_WIDTH = 25
        self.WIDGET_APPLICATION_TOTAL_WIDTH = 260
        self.WIDGET_APPLICATION_TOTAL_HEIGHT = 295
        self.WIDGET_CHANNEL_DIALOG_WIDTH = 400
        self.WIDGET_CHANNEL_TOTAL_WIDTH = 300
        self.WIDGET_CHANNEL_TOTAL_HEIGHT = 40
        self.WIDGET_CHANNEL_PADDING = 5
        self.WIDGET_RUNNING_APPS_WIDTH = 450
        self.WIDGET_RUNNING_APPS_TOTAL_WIDTH = 350
        self.WIDGET_RUNNING_APPS_TOTAL_HEIGHT = 55
        self.WIDGET_RUNNING_APPS_PADDING = 10
        self.WIDGET_LOGIN_CARD_TOTAL_WIDTH = 315
        self.WIDGET_LOGIN_CARD_TOTAL_HEIGHT = 200

    def __repr__(self):
        """Return a pretty formtated representation of the enum."""
        keys = []
        representation = 'SASS variables enum: \n'
        for key in self.__dict__:
            if key[0] in string.ascii_uppercase:
                keys.append(key)

        for key in sorted(keys):
            representation += '    {0} = {1}\n'.format(key, self.__dict__[key])
        return representation


SASS_VARIABLES = SassVariables()


def load_sass_variables(data):
    """Parse Sass file styles and get custom values for used in code."""
    global SASS_VARIABLES
    pattern = re.compile(r'[$]\S*:.*?;')
    variables = re.findall(pattern, data)
    for var in variables:
        name, value = var[1:-1].split(':')
        if name[0] in string.ascii_uppercase:
            value = value.strip()
            try:
                value = ast.literal_eval(value)
            except Exception:
                pass
            setattr(SASS_VARIABLES, name, value)
    return SASS_VARIABLES


def load_style_sheet():
    """Load css styles file and parse to include custom variables."""
    with open(GLOBAL_SASS_STYLES_PATH, 'rt') as f:
        sass_data = f.read()

    load_sass_variables(sass_data)

    with open(GLOBAL_STYLES_PATH, 'rt') as f:
        data = f.read()

    if os.name == 'nt':
        data = data.replace('$IMAGE_PATH', images.IMAGE_PATH.replace('\\', '/'))
    else:
        data = data.replace('$IMAGE_PATH', images.IMAGE_PATH)

    return data



# Standard library imports
import json
import os
import stat
import traceback
import typing
import urllib.parse
import uuid
import collections
from datetime import datetime
from os.path import exists, isdir, isfile, join

# Third party imports
import requests
import repo_cli
from binstar_client.utils.config import TOKEN_DIRS

from anaconda_navigator.utils.logs import logger

try:
    from urllib import quote_plus
except ImportError:
    from urllib.parse import quote_plus


Args = collections.namedtuple('Args', ['site'])


def inject_url_query(url: str, arguments: typing.Mapping[str, typing.Any]) -> str:
    """Replace query part in the `url`."""
    scheme: str
    netloc: str
    path: str
    params: str
    query: str
    fragment: str

    scheme, netloc, path, params, query, fragment = urllib.parse.urlparse(url)
    query = urllib.parse.urlencode(arguments)
    return urllib.parse.urlunparse((scheme, netloc, path, params, query, fragment))


class TeamEditionAPI:
    """
    This is the interface for working with Anaconda Team Edition API.
    """
    DEFAULT_CONTENT_TYPE = {'Content-Type': 'application/json'}

    def __init__(self, base_url, config, verify_ssl=False):

        self._urls = {
            'repo': '{}/api/repo'.format(base_url),
            'login': '{}/api/auth/login'.format(base_url),
            'account': '{}/api/account'.format(base_url),
            'tokens': '{}/api/account/tokens'.format(base_url),
            'channels': '{}/api/channels'.format(base_url)
        }
        self._config = config
        self._verify_ssl = verify_ssl
        self.token = None
        self.domain = base_url

    def authenticate(self, username, password):
        """
        Login using direct grant and returns the JWT token.

        :param str username: The username to authenticate.
        :param str password: The password to authenticate.

        :return str: The dumped json string.
        """
        data = {'username': username, 'password': password}
        s = requests.Session()

        try:
            resp = s.post(
                self._urls['login'], data=json.dumps(data), headers=self.DEFAULT_CONTENT_TYPE, verify=self._verify_ssl
            )

        except requests.exceptions.SSLError:
            return json.dumps(
                {
                    'message':
                    'Your SSL certificate is self-signed or broken and cannot be validated. '
                    'Please change the certificate or set “ssl_verify:false” '
                    'in your .condarc configuration file.'
                }
            )

        except Exception:
            logger.error(
                'Exception happened during the login into Team Edition. Traceback: {}'.format(traceback.format_exc())
            )
            return json.dumps(
                {
                    'message':
                    'An unexpected error happened! Please see logs at {} and '
                    'contact your system administrator.'.format(logger.handlers[0].baseFilename)
                }
            )

        else:
            if resp.status_code != 200:
                return json.dumps(resp.json())

            token = resp.json()
            token['jwt_token'] = token['token']
            token['jwt_token_refresh'] = token['refresh_token']

            return json.dumps(token)

    def logout(self):
        self.remove_token()

    def _get_team_edition_api_url(self):
        return 'jwt_{}'.format(self._config.get('main', 'team_edition_api_url'))

    def load_token(self):
        """
        Loads the JWT token to be used in further requests to authenticate user.

        :return str:
        """
        for token_dir in TOKEN_DIRS:
            url = self._get_team_edition_api_url()

            tokenfile = join(token_dir, '%s.token' % quote_plus(url))
            _is_file = exists(tokenfile)

            if _is_file:
                with open(tokenfile) as fd:
                    token = fd.read().strip()

                if token:
                    return token
                else:
                    os.unlink(tokenfile)

        return "{}"

    def store_binstar_token(self, token):
        """
        Stores the binstar token required to work with Team Edition
        through 'native' conda interface.

        :param str token: The string as a token.
        """
        for token_dir in TOKEN_DIRS:
            if not isdir(token_dir):
                os.makedirs(token_dir)
            tokenfile = join(token_dir, '%s.token' % quote_plus(self._urls['repo']))

            if isfile(tokenfile):
                os.unlink(tokenfile)

            with open(tokenfile, 'w') as fd:
                fd.write(token)
            os.chmod(tokenfile, stat.S_IWRITE | stat.S_IREAD)

    def store_token(self, token):
        """
        Stores the passed JWT token locally to be used in further
        user authentication.

        :param str token: The json data dumped in the string.
        """
        for token_dir in TOKEN_DIRS:
            url = self._get_team_edition_api_url()

            if not isdir(token_dir):
                os.makedirs(token_dir)
            tokenfile = join(token_dir, '%s.token' % quote_plus(url))

            if isfile(tokenfile):
                os.unlink(tokenfile)

            with open(tokenfile, 'w') as fd:
                fd.write(token)
            os.chmod(tokenfile, stat.S_IWRITE | stat.S_IREAD)

    def remove_token(self):
        """
        Removes the existing JWT token from local space.
        """
        jwt_url = self._get_team_edition_api_url()
        url = self._get_team_edition_api_url().replace('jwt_', '')

        url_suffixes = [
            url,
            jwt_url,
            '{}/{}'.format(jwt_url, 'repo'),
            '{}/{}'.format(url, 'repo'),
            '{}/{}'.format(url, 'api'),
            '{}/{}/{}'.format(url, 'api', 'repo')
        ]

        for token_dir in TOKEN_DIRS:
            for url_suffix in url_suffixes:
                tokenfile = join(token_dir, '%s.token' % quote_plus(url_suffix))
                if isfile(tokenfile):
                    os.unlink(tokenfile)
        try:
            args = Args(None)
            repo_cli.utils.config.remove_token(args)
        except TypeError:
            pass

    def user(self, login=None):
        """
        Gets the user account info.

        :return dict[str, str]: The user data.
        """
        token = json.loads(self.load_token())

        s = requests.Session()
        resp = s.get(self._urls['account'], headers=self.DEFAULT_CONTENT_TYPE, cookies=token, verify=self._verify_ssl)

        data = resp.json()

        return {
            'login': data['username'],
        }

    def create_access_token(self, token):
        """
        Creates the access token (binstar) for the Team Edition API to provide
        access to channels listed in the repo.

        Returns the token and the token id.

        :param dict[str, str] token: The JWT token to be used for authentication.

        :return dict: The access token info.
        """
        request_data = {
            'name':
            'navigator-token-{}-{}'.format(datetime.now().strftime('%Y-%m-%d'), uuid.uuid4()),
            'expires_at':
            '{}-12-31'.format(datetime.now().year),
            'scopes': [
                'channel:view', 'channel:view-artifacts', 'subchannel:view', 'subchannel:view-artifacts',
                'artifact:view', 'artifact:download'
            ]
        }

        s = requests.Session()
        resp = s.post(
            self._urls['tokens'],
            headers=self.DEFAULT_CONTENT_TYPE,
            data=json.dumps(request_data),
            cookies=token,
            verify=self._verify_ssl
        )

        response_data = resp.json()
        self.store_binstar_token(response_data.get('token'))

        return response_data

    def remove_access_token(self, access_token_id):
        """
        Deletes the created access token (binstar) from the Team Edition database.

        :param str access_token_id: The token ID to be removed.
        """
        token = json.loads(self.load_token())

        s = requests.Session()
        s.delete(
            '{}/{}'.format(self._urls['tokens'], access_token_id),
            headers=self.DEFAULT_CONTENT_TYPE,
            cookies=token,
            verify=self._verify_ssl
        )

    def get_channels(self) -> typing.List[typing.Dict[str, typing.Any]]:
        """
        Returns all the channels which are accessible for the user.

        :rtype: list[dict[str, mixed]]
        :return: List of dictionaries with available channels data.
        """
        token: typing.Mapping[str, typing.Any] = json.loads(self.load_token())

        headers: typing.Mapping[str, typing.Any]
        if 'token' in token:
            headers = {'Authorization': 'Bearer {}'.format(token['token'])}
        else:
            headers = {'X-Auth': self.token}

        result: typing.List[typing.Dict[str, typing.Any]] = []

        step: int = 1000
        offset: int = 0
        total: typing.Optional[int] = None
        while (total is None) or (offset < total):
            url: str = inject_url_query(
                self._urls['channels'], {'offset': offset, 'limit': step, 'sort': 'name,updated_at'},
            )
            response: requests.Response = requests.get(url, headers=headers)
            response.raise_for_status()

            content: typing.Mapping[str, typing.Any] = response.json()
            if total is None:
                total = content['total_count']
            result.extend(content['items'])

            offset += step

        return result



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""Anaconda Cloud login dialog."""

from .account_dialogs import *  # noqa: F403
from .anaconda_dialogs import *  # noqa: F403
from .commercial_dialogs import *  # noqa: F403
from .enterprise_dialogs import *  # noqa: F403
from .nucleus_dialogs import *  # noqa: F403
from .team_dialogs import *  # noqa: F403


# --- Local testing
# -----------------------------------------------------------------------------
def local_test():  # pragma: no cover
    """Run local test."""
    from anaconda_navigator.static.fonts import load_fonts
    from anaconda_navigator.utils.qthelpers import qapplication
    app = qapplication(test_time=3)
    load_fonts(app)

    # ENTERPRISE EDITION TESTING
    widgets_iter = (EnterpriseRepoSetDomainPage, EnterpriseRepoLoginPage, NoticePage)  # noqa: F405
    for widget in widgets_iter:
        app.w = widget()
        exit_code = app.w.exec()
        print(exit_code)
        if not exit_code:
            break


if __name__ == '__main__':  # pragma: no cover
    local_test()
